/*
 * Routines for dealing with resizing GEM windows. Called when a resize
 * request has been made for window 'win'
 */

#include "gemx.h"

void GEMresizeWindow(int win)
	{
	WORD newW, newH;
	WORD oldX, oldY, oldW, oldH;

	oldX = GEMwins[win].ax;
	oldY = GEMwins[win].ay;
	oldW = GEMwins[win].aw;
	oldH = GEMwins[win].ah;


	graf_rubberbox(oldX, oldY, oldW, oldH, &newW, &newH);
	newW += SLIDSZ * (GEMwins[win].bits | VSLIDE);
	newH += SLIDSZ * (GEMwins[win].bits | NAME);
	newH += SLIDSZ * (GEMwins[win].bits | INFO);
	newH += SLIDSZ * (GEMwins[win].bits | HSLIDE);

	GEMdoResize(win, newW, newH);
	}

void GEMdoResize(int win, int newW, int newH)
	{
	int Parts = GEMwins[win].bits;

	int addTx, addTy, addTw, addTh;		/* co-ords of the topbar		*/
	int addIx, addIy, addIw, addIh;		/* co-ords of the infobar		*/
	int addVx, addVy, addVw, addVh;		/* co-ords of the V. slider		*/
	int addHx, addHy, addHw, addHh;		/* co-ords of the H. slider		*/

	int addH, addW;		/* additions to parent window due to borders	*/

	/* Clip to the maximum width & height of the window					*/
	newW = (newW > GEMwins[win].mw)? GEMwins[win].mw: newW;
	newH = (newH > GEMwins[win].mh)? GEMwins[win].mh: newH;

	/* Set the additions = to 0											*/
	addH = addW =0;

	/* See if we should add a title bar									*/
	addTx = addTy = addTw = addTh =0;
	if (Parts & NAME)
		{
		GEMwins[win].topSpace = SLIDSZ;
		addTx = -1;
		addTy = 0;
		addTw = newW;
		addTh = SLIDSZ;
		addH += 1;

		/* See if there is a close button								*/
		if (Parts & CLOSER)
			{
			addTx += SLIDSZ;
			addTw -= SLIDSZ;
			}

		/* See if there is a full button								*/
		if (Parts & FULLER)
			addTw -= SLIDSZ;
		}
	else
		GEMwins[win].topSpace = 0;



	/* See if we should add an info bar									*/
	if (Parts & INFO)
		{
		GEMwins[win].infSpace = SLIDSZ;
		addIx = -1;
		addIy = SLIDSZ;
		addIw = newW;
		addIh = SLIDSZ;
		addH += 1;
		}
	else
		GEMwins[win].infSpace = 0;



	/* See if we should add vertical sliderbars							*/
	if (Parts & VSLIDE)
		{
		GEMwins[win].sideSpace = SLIDSZ;
		addVx = newW;
		addVy = addH*SLIDSZ;
		addVw = SLIDSZ;
		addVh = newH;
		addW += 1;

		/* Correct the titlebar and infobar widths						*/
		addIw += SLIDSZ;
		addTw += SLIDSZ;

		/* See if we need to add a scrollup arrow						*/
		if (Parts & UPARROW)
			{
			addVy += SLIDSZ;
			addVh -= SLIDSZ;
			}

		/* See if we need to add a scrolldown arrow						*/
		if (Parts & DNARROW)
			{
			addVh -= SLIDSZ;
			}

		}
	else
		GEMwins[win].sideSpace = 0;



	/* See if we should add horizontal sliderbars						*/
	if (Parts & HSLIDE)
		{
		GEMwins[win].botSpace = SLIDSZ;
		addHx = -1;
		addHy = newH + addH*SLIDSZ+1;
		addHw = newW + addW*SLIDSZ;
		addHh = SLIDSZ;
		addH += 1;

		
		/* Check for a resizer box										*/
		GEMwins[win].sizerX = newW + addW*SLIDSZ -SLIDSZ;
		if (Parts & SIZER)
			{
			addHw -= SLIDSZ;
			}

		/* Check for the right arrow									*/
		if (Parts & RTARROW)
			{
			addHw -= SLIDSZ;
			GEMwins[win].rtArrX = GEMwins[win].sizerX - SLIDSZ;
			}

		/* Check for the left arrow										*/
		if (Parts & LFARROW)
			{
			addHw -= SLIDSZ;
			addHx += SLIDSZ;
			}
			
		}
	else
		GEMwins[win].botSpace =0;


	/* So we now have co-ords of the window parts						*/
	GEMwins[win].px = GEMwins[win].ax;
	GEMwins[win].py = GEMwins[win].ay;
	GEMwins[win].pw = GEMwins[win].aw;
	GEMwins[win].ph = GEMwins[win].ah;

	GEMwins[win].aw = newW + GEMwins[win].sideSpace + addW;
	GEMwins[win].ah = newH + GEMwins[win].topSpace +
						GEMwins[win].botSpace +
						GEMwins[win].infSpace + addH;

	/* Now go about the resizing/moving ops								*/
	XResizeWindow(display, GEMwins[win].win, GEMwins[win].aw,
						GEMwins[win].ah);

	XResizeWindow(display, GEMwins[win].topBar, addTw, addTh);
	XMoveWindow(display, GEMwins[win].fuller, addTx+ addTw, 0);

	XResizeWindow(display, GEMwins[win].infoBar, addIw, addIh);

	XMoveResizeWindow(display, GEMwins[win].sidBar, addVx, addVy, addVw, addVh);
	XMoveWindow(display, GEMwins[win].upArrow, addVx, addVy -SLIDSZ);
	XMoveWindow(display,GEMwins[win].dnArrow, addVx, addVy + addVh +1);

	XMoveResizeWindow(display, GEMwins[win].botBar,addHx, addHy, addHw, addHh);
	XMoveWindow(display, GEMwins[win].ltArrow, -1, addHy);
	XMoveWindow(display, GEMwins[win].rtArrow,
						GEMwins[win].rtArrX, addHy);
	XMoveWindow(display, GEMwins[win].sizer,
						GEMwins[win].sizerX, addHy);
	XResizeWindow(display, GEMwins[win].client, newW, newH);
	}
