/*
 * appl.c : a translation from the GEM application library interface to the
 *          X protocol.
 *
 * Last modified	: Mon Sep 13 16:48:48 WET DST 1993
 * Author			: sjg@unhygeinix.ph.kcl.ac.uk
 *
 *
 * Contains replacements for:
 *
 * short appl_init()
 * short appl_read(int ApId, int Length, void *ApPbuff)
 * short appl_write(int ApId, int Length, void *ApPbuff)
 * short appl_find(const char *Name)
 * short appl_tplay(void *Mem, int Num, int Scale)
 * short appl_trecord(void *Mem, int Count)
 * short appl_exit()
 *
 */

#define __APPL_H_GLOBALS

#include <sys/types.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <signal.h>

#include "gemx.h"


void replace(char *string, char old, char new);

WORD appl_init()
	{
	/* returns a GEMid unique to this program. Makes sure that the GEM
	 * programs running on one machine can communicate. This comms ought
	 * really be done with sockets... well - one day :-)
	 *
	 * Sets up an X window that constitutes the desktop background...
	 */

	XSetWindowAttributes attrib;	/* the desktops window attributes	*/
	XSizeHints hints;				/* Hints to the window manager		*/
	long attribMask;				/* attribute mask 					*/
	char **argv;
	int argc =0;

	DIR *dp;						/* for parsing the Gem-X directory	*/
	struct dirent *entry;			/* ditto							*/
	int max = -1;					/* ditto							*/

	char buffer[1024];				/* general buffer					*/

	int i;							/* Loop variable					*/

	/* Connect to the X server											*/
	if ((display=XOpenDisplay(NULL)) == NULL)
		fatalError("application","Can't open X display");

	/* fetch some information from the X server							*/
	screen_num = DefaultScreen(display);
	GEMwd = DisplayWidth(display, screen_num);
	GEMht = DisplayHeight(display, screen_num);
	GEMdp = DefaultDepth(display, screen_num);

	/* set up the initial values for the GEM variables					*/
	GEMcountW = -1;
	GEMDeskW = 800;
	GEMDeskH = 600;


	/* DEBUG: */
	/* XSynchronize(display, 1); */


	/* make sure the directory GEMXDIR exists							*/
	if ((dp = opendir(GEMXDIR)) == NULL)
		{
		char buffer[1024];
		sprintf(buffer, "mkdir -p %s", GEMXDIR);
		system(buffer);
		dp = opendir(GEMXDIR);
		}

	/* get rid of . and ..												*/
	entry = readdir(dp);
	entry = readdir(dp);

	/* Now get the highest GEMid entry...								*/
	for (entry = readdir(dp); entry != NULL; entry = readdir(dp))
		{
		/* This loop is entered if there are any GEM programs running
		 * already.
		 */

		char name[1024];
		int id;
		strncpy(name, entry->d_name, entry->d_namlen);
		replace(name, ':', ' ');
		sscanf(name, "%s %d", buffer, &id);
		max = (max>id)? max: id;
		}

	/* if max == -1 then we have no GEM procs running. So set max =0	*/
	GEMapId = max ++;

	/* create the GEMid file in the GEMXDIR								*/
	sprintf(buffer, "%s/gemprog:%d", GEMXDIR, max);
	if ((mkfifo(buffer, S_IRWXU | S_IRWXG | S_IRWXO)) == -1)
		fatalError("application", "Could not create Gem-X fifo");

	/* Set up the attributes and hints for the window manager			*/
	attrib.background_pixmap = ParentRelative;
	attribMask |= CWBackPixmap;
	hints.x = 100;
	hints.y = 200;
	hints.flags = PPosition | PSize;
	
	/* Create a desktop X window to parent all the GEM windows			*/
	GEMDesk = XCreateWindow(display,
					RootWindow(display, screen_num),
					100, 200, GEMDeskW, GEMDeskH,
					2, GEMdp, InputOutput, visual, attribMask, &attrib);
	XMapWindow(display, GEMDesk);
	XMapSubwindows(display, GEMDesk);
	XSelectInput(display, GEMDesk, StructureNotifyMask | KeyPressMask |
					ExposureMask | ButtonPressMask |
					PointerMotionMask | ButtonReleaseMask);

	/* create a GC for the entire GEM system							*/
	gc[0] = XCreateGC(display, GEMDesk, 0,0);
	GCmap[0] = True;

	/* Set the GC count to 1 - this is the first USER GC				*/
	GEMwks = 1;


	/* initialise the GC defaults										*/
	for (i=1; i<GEMworks; i++)
		{
		GCwid[i]  = 0;
		GCmap[i]  = 0;
		GCend[i]  = CapButt;
		GCdash[i] = LineSolid;
		}

	/* initialise the window defaults									*/
	for (i=0; i<GEMwindMax; i++)
		GEMwinList[i] = -1;


	XSetForeground(display, gc[0], BlackPixel(display, screen_num));
	XSetBackground(display, gc[0], WhitePixel(display, screen_num));

	/* Create the pixmap for the desktop background						*/
	createPixmaps();

	XSetWindowBackgroundPixmap(display, GEMDesk, GEMBack);
	XClearWindow(display, GEMDesk);
	XSetStandardProperties(display, GEMDesk, "Desktop", "GEMdesk",
				GEMIcon, argv, argc, &hints);
	return (max);
	}

void replace(char *string, char old, char new)
	{
	int i;

	for (i=0; i<strlen(string); i++)
		string[i] = (string[i] == old)? new : string[i];
	} 


WORD appl_exit()
	{
	int i;

	/* terminates the program, and cleans up after it					*/
	char buffer[1024];

	/* remove the fifo file												*/
	sprintf(buffer, "%s/gemprog.%d", GEMXDIR, GEMapId);

	/* Be nice to the X server - remove all the pixmaps...				*/
	freePixmaps();

	/* release the GC's 												*/
	for (i=0; i<10; i++)
		if (GCmap[i])
			XFreeGC(display, gc[i]);

	exit(0);
	}
