/*
 * routines to handle the graf_* library in the AES
 */

#include "gemx.h"

WORD graf_rubberbox(WORD x, WORD y, WORD w, WORD h, WORD *oW, WORD *oH)
	{
	/* takes a (x,y,w,h) co-ord set, and returns oW and oH when finished
	 * Rubberbands a box at the co-ords until the mouse button is raised
	 * Co-ords are taken to be relative to the desktop window
	 */

	int i, j;
	XEvent event;

	XSetFunction(display, gc[0], GXxor);
	XSetSubwindowMode(display, gc[0], IncludeInferiors);
	XDrawRectangle(display, GEMDesk, gc[0], x, y, w, h);

	while (XCheckTypedEvent(display, ButtonRelease, &event) == False)
		{
		Window root, child;
		int rtX, rtY, wnX, wnY;
		unsigned int mask;

		/* Find where the pointer is at the moment						*/
		XQueryPointer(display, GEMDesk, &root, &child, 
					&rtX, &rtY, &wnX, &wnY, &mask);

		if ( (wnX != x) || (wnY != y) )
			{
			/* we've moved, so rub out the box, and redraw				*/
			XDrawRectangle(display, GEMDesk, gc[0], x, y, w, h);
			w=(wnX -x > 0)? wnX -x: 0;
			h=(wnY -y > 0)? wnY -y: 0;
			XDrawRectangle(display, GEMDesk, gc[0], x, y, w, h);
			}
		}


	/* Rub out the rectangle, and return the values						*/
	XDrawRectangle(display, GEMDesk, gc[0], x, y, w, h);
	XSetFunction(display, gc[0], GXcopy);

	*oW = w;
	*oH = h;

	return 1;
	}


graf_dragbox(WORD sW, WORD sH, WORD sX, WORD sY,
			 WORD bX, WORD bY, WORD bW, WORD bH,
			 WORD *fX, WORD *fY)
	{
	int xOff, yOff;
	Window root, child;
	int rtX, rtY, wnX, wnY;
	int x,y;
	unsigned int mask;
	XEvent event;

 	XSetFunction(display, gc[0], GXxor);
	XSetSubwindowMode(display, gc[0], IncludeInferiors);

	/* Find the offsets for the top of the rectangle from the mouse		*/
	XQueryPointer(display, GEMDesk, &root, &child, &rtX, &rtY,
				&wnX, &wnY, &mask);
	xOff = sX - wnX;
	yOff = sY - wnY;
	x=sX;
	y=sY;

	XDrawRectangle(display, GEMDesk, gc[0], x, y, sW, sH);

	while (XCheckTypedEvent(display, ButtonRelease, &event) == False)
		{
		/* Find where the pointer is at the moment						*/
		XQueryPointer(display, GEMDesk, &root, &child, 
					&rtX, &rtY, &wnX, &wnY, &mask);

		x = wnX + xOff;
		y = wnY + yOff;

		if ( (x != sX ) || (y != sY) )
			{
			/* we've moved, so rub out the box, and redraw				*/
			XDrawRectangle(display, GEMDesk, gc[0], sX, sY, sW, sH);

			/* Bind the rectangle to a clipping rectangle				*/
			sX = x = (wnX +xOff > bX)? wnX +xOff: bX;
			sY = y = (wnY +yOff > bY)? wnY +yOff: bY;
			sX = x = (wnX +xOff + sW < bW + bX)? x: bW + bX - sW;
			sY = y = (wnY +yOff + sH < bH + bY)? y: bH + bY - sH;

			XDrawRectangle(display, GEMDesk, gc[0], x, y, sW, sH);
			}
		}


	/* Rub out the rectangle, and return the values						*/
	XDrawRectangle(display, GEMDesk, gc[0], x, y, sW, sH);
	XSetFunction(display, gc[0], GXcopy);

	*fX = x;
	*fY = y;

	return 1;
	}
