/*
 * pixmaps.c: source to load and convert into pixmaps all the bitmaps that I
 * use for the GEM emulation
 */

#include "gemx.h"

/* buffer used in this source file for string operations				*/
char buffer[1024];
char buffr1[1024];

/* 'private' function prototypes...										*/
Pixmap createBitmap(char *name, int *width, int *height);
Pixmap createPixmap(char *name);
	
createPixmaps()
	{
	int width, height;
	int i,j;

	GEMBack  = createPixmap("background.xbm");
	GEMIcon  = createBitmap("gremlin.xbm", &width, &height);

	GEMarrU  = createPixmap("up.xbm");
	GEMarrD  = createPixmap("down.xbm");
	GEMarrL  = createPixmap("left.xbm");
	GEMarrR  = createPixmap("right.xbm");
	GEMclose = createPixmap("close.xbm");
	GEMsize  = createPixmap("resize.xbm");
	GEMfull  = createPixmap("full.xbm");
	GEMwhite = createPixmap("white.xbm");

	for (i=1; i<=24; i++)
		{
		sprintf(buffr1, "fill.%d.xbm", i);
		GEMPatFl[i] = createPixmap(buffr1);
		}

	for (i=1; i<=12; i++)
		{
		sprintf(buffr1, "fll.%d.xbm", i);
		GEMLinFl[i] = createPixmap(buffr1);
		}
	}

freePixmaps()
	{
	int i;

	XFreePixmap(display, GEMBack);
	XFreePixmap(display, GEMIcon);
	XFreePixmap(display, GEMarrU);
	XFreePixmap(display, GEMarrD);
	XFreePixmap(display, GEMarrL);
	XFreePixmap(display, GEMarrR);
	XFreePixmap(display, GEMclose);
	XFreePixmap(display, GEMfull);
	XFreePixmap(display, GEMsize);

	for (i=1; i<=24; i++)
		XFreePixmap(display, GEMPatFl[i]);

	for (i=1; i<=12; i++)
		XFreePixmap(display, GEMLinFl[i]);
	}


Pixmap createBitmap(char *name, int *width, int *height)
	{
	/* reads in the appropriate bitmap filename and returns a Pixmap
	 * of depth 1 bitplane
	 */

	Pixmap thePix;
	int xHot, yHot;
 
	/* create the filename												*/
	sprintf(buffer, "%s/%s", GEMXLIB, name);

	/* and create the bitmap											*/
	XReadBitmapFile(display, GEMDesk, buffer, width, height,
				&thePix,  &xHot, &yHot);

	return thePix;
	}
	
Pixmap createPixmap(char *name)
	{
	/* reads in a bitmap and assigns it to a depth of Pixmap =to GEMdp	*/

	Pixmap thePix;
	Pixmap theBit;

	int width, height;

	/* Read in the bitmap 												*/
	theBit = createBitmap(name, &width, &height);

	/* Now copy it to a pixmap of depth GEMdp							*/
	thePix = XCreatePixmap(display, GEMDesk, width, height, GEMdp);
	XCopyPlane(display, theBit, thePix, gc[0], 0, 0, width, height, 
				0, 0, 1);

	return thePix;
	}


	
