 /* wind.c : a translation from the GEM window library interface to the
 *          X protocol.
 *
 * Last modified	: Thu Sep 23 17:55:15 WET DST 1993
 * Author			: sjg@unhygeinix.ph.kcl.ac.uk
 *
 *
 * Contains replacements for:
 *
 *WORD wind_create(int Parts, int Wx, int Wy, int Ww, int Wh)
 *WORD wind_open(int Handle, int Wx, int Wy, int Ww,int Wh)
 *WORD wind_close(int Handle)
 *WORD wind_delete(int Handle)
 *WORD wind_get(WORD Handle, WORD What, WORD *W1, WORD *W2, WORD *W3, WORD *W4)
 *WORD wind_set(WORD Handle, WORD What, WORD  W1, WORD  W2, WORD  W3, WORD  W4)
 *WORD wind_find(WORD X, WORD Y)
 *WORD wind_update(WORD code)
 *WORD wind_calc(WORD type, WORD parts, WORD x, WORD y, WORD w, WORD h,
                                WORD *ox, WORD *oy, WORD *ow, WORD *oh)
 */


#define __WIND_H_GLOBALS
#define DEBUG 1

#include "gemx.h"

int calcW(int Handle, int clientW);
int calcH(int Handle, int clientH);
Window createWindow(Window parent, int x, int y, int w, int h);


WORD wind_create(LONG Parts, WORD Wx, WORD Wy, WORD Ww, WORD Wh)
	{
	XSetWindowAttributes attrib;
	long attribMask;

	int addTx, addTy, addTw, addTh;		/* co-ords of the topbar		*/
	int addIx, addIy, addIw, addIh;		/* co-ords of the infobar		*/
	int addVx, addVy, addVw, addVh;		/* co-ords of the V. slider		*/
	int addHx, addHy, addHw, addHh;		/* co-ords of the H. slider		*/

	int addH, addW;		/* additions to parent window due to borders	*/

	/* Set the additions = to 0											*/
	addH = addW =0;

	/* increment our window counter										*/
	GEMcountW ++;



	/* See if we should add a title bar									*/
	addTx = addTy = addTw = addTh =0;
	if (Parts & NAME)
		{
		GEMwins[GEMcountW].topSpace = SLIDSZ;
		addTx = -1;
		addTy = 0;
		addTw = Ww;
		addTh = SLIDSZ;
		addH += 1;

		/* See if there is a close button								*/
		if (Parts & CLOSER)
			{
			addTx += SLIDSZ;
			addTw -= SLIDSZ;
			}

		/* See if there is a full button								*/
		if (Parts & FULLER)
			addTw -= SLIDSZ;
		}
	else
		GEMwins[GEMcountW].topSpace = 0;



	/* See if we should add an info bar									*/
	if (Parts & INFO)
		{
		GEMwins[GEMcountW].infSpace = SLIDSZ;
		addIx = -1;
		addIy = SLIDSZ;
		addIw = Ww;
		addIh = SLIDSZ;
		addH += 1;
		}
	else
		GEMwins[GEMcountW].infSpace = 0;



	/* See if we should add vertical sliderbars							*/
	if (Parts & VSLIDE)
		{
		GEMwins[GEMcountW].sideSpace = SLIDSZ;
		addVx = Ww;
		addVy = addH*SLIDSZ;
		addVw = SLIDSZ;
		addVh = Wh;
		addW += 1;

		/* Correct the titlebar and infobar widths						*/
		addIw += SLIDSZ;
		addTw += SLIDSZ;

		/* See if we need to add a scrollup arrow						*/
		if (Parts & UPARROW)
			{
			addVy += SLIDSZ;
			addVh -= SLIDSZ;
			}

		/* See if we need to add a scrolldown arrow						*/
		if (Parts & DNARROW)
			{
			addVh -= SLIDSZ;
			}

		}
	else
		GEMwins[GEMcountW].sideSpace = 0;



	/* See if we should add horizontal sliderbars						*/
	if (Parts & HSLIDE)
		{
		GEMwins[GEMcountW].botSpace = SLIDSZ;
		addHx = -1;
		addHy = Wh + addH*SLIDSZ;
		addHw = Ww + addW*SLIDSZ;
		addHh = SLIDSZ;
		addH += 1;

		
		/* Check for a resizer box										*/
		GEMwins[GEMcountW].sizerX = Ww + addW*SLIDSZ -SLIDSZ;
		if (Parts & SIZER)
			{
			addHw -= SLIDSZ;
			}

		/* Check for the right arrow									*/
		if (Parts & RTARROW)
			{
			addHw -= SLIDSZ;
			GEMwins[GEMcountW].rtArrX = GEMwins[GEMcountW].sizerX - SLIDSZ;
			}

		/* Check for the left arrow										*/
		if (Parts & LFARROW)
			{
			addHw -= SLIDSZ;
			addHx += SLIDSZ;
			}
			
		}
	else
		GEMwins[GEMcountW].botSpace =0;





	/* Now put the given values into the window structure...			*/
	GEMwins[GEMcountW].bits = Parts;

	GEMwins[GEMcountW].px = 0;
	GEMwins[GEMcountW].py = 0;
	GEMwins[GEMcountW].pw = GEMDeskW;
	GEMwins[GEMcountW].ph = GEMDeskH;

	GEMwins[GEMcountW].ax = GEMwins[GEMcountW].mx = Wx;
	GEMwins[GEMcountW].ay = GEMwins[GEMcountW].my = Wy;
	GEMwins[GEMcountW].aw = GEMwins[GEMcountW].mw = Ww;
	GEMwins[GEMcountW].ah = GEMwins[GEMcountW].mh = Wh;

	/* add the bits on to the parent window size values					*/
	GEMwins[GEMcountW].aw += GEMwins[GEMcountW].sideSpace + addW;
	GEMwins[GEMcountW].ah += GEMwins[GEMcountW].topSpace +
							 GEMwins[GEMcountW].botSpace +
							 GEMwins[GEMcountW].infSpace + addH;

	/* Now create an X parent window big enough to hold the bits		*/
	GEMwins[GEMcountW].win = createWindow(GEMDesk,
					GEMwins[GEMcountW].ax, GEMwins[GEMcountW].ay,
					GEMwins[GEMcountW].aw, GEMwins[GEMcountW].ah);
	XSelectInput(display, GEMwins[GEMcountW].win,  ExposureMask |
					ButtonPressMask | ButtonReleaseMask | KeyPressMask |
					PointerMotionMask | StructureNotifyMask);

	/* Add in the topbar if we need to...								*/
	if (Parts & NAME)
		{
		GEMwins[GEMcountW].topBar = createWindow(
					GEMwins[GEMcountW].win,
					addTx, addTy, addTw, addTh);

		/* and see if we should add a closer box						*/
		if (Parts & CLOSER)
			GEMwins[GEMcountW].closer = createWindow(
						GEMwins[GEMcountW].win,
						-1, 0, SLIDSZ, SLIDSZ);

		/* and see if we should add a fuller box						*/
		if ( Parts & FULLER )
			GEMwins[GEMcountW].fuller = createWindow(
						GEMwins[GEMcountW].win,
						addTx+ addTw, 0, SLIDSZ, SLIDSZ);
		}

	if (Parts & INFO)
		GEMwins[GEMcountW].infoBar = createWindow(
					GEMwins[GEMcountW].win,
					addIx, addIy, addIw, addIh);


	/* Add in the vertical sliderbar if we need to 						*/
	if (Parts & VSLIDE)
		{
		GEMwins[GEMcountW].sideHt = addVh -1;
		GEMwins[GEMcountW].sidBar = createWindow(
					GEMwins[GEMcountW].win,
					addVx, addVy, addVw, addVh);			


		/* See if we need to add the up arrow box						*/
		if (Parts & UPARROW)
			GEMwins[GEMcountW].upArrow = createWindow(
						GEMwins[GEMcountW].win,
						addVx, addVy -SLIDSZ, SLIDSZ, SLIDSZ);

		/* See if we need to add the down arrow box						*/
		if (Parts & DNARROW)
			GEMwins[GEMcountW].dnArrow = createWindow(
						GEMwins[GEMcountW].win,
						addVx, addVy + addVh +1, SLIDSZ, SLIDSZ);
		}

	/* Add in the horizontal sliderbar if we need to 					*/
	if (Parts & HSLIDE)
		{
		GEMwins[GEMcountW].botWd  = addHw -1;
		GEMwins[GEMcountW].botBar = createWindow(
					GEMwins[GEMcountW].win,
					addHx, addHy, addHw, addHh);

		/* See if we need to add the left arrow box						*/
		if (Parts & LFARROW)
			GEMwins[GEMcountW].ltArrow = createWindow(
						GEMwins[GEMcountW].win,
						-1, addHy, SLIDSZ, SLIDSZ);

		/* See if we need to add the right arrow box					*/
		if (Parts & RTARROW)
			GEMwins[GEMcountW].rtArrow = createWindow(
						GEMwins[GEMcountW].win,
						GEMwins[GEMcountW].rtArrX, addHy, SLIDSZ, SLIDSZ);

		/* See if we need to add the sizer box							*/
		if (Parts & SIZER)
			GEMwins[GEMcountW].sizer = createWindow(
						GEMwins[GEMcountW].win,
						GEMwins[GEMcountW].sizerX, addHy +1, SLIDSZ, SLIDSZ);

		}
	/* Add the client window to the structure...						*/
	GEMwins[GEMcountW].client = createWindow(
					GEMwins[GEMcountW].win,
					-1, addVy -SLIDSZ, Ww, Wh);

	/* add this window to the window list								*/
	GEMwinList[GEMcountW] = GEMcountW;
	return GEMcountW; 
	}

WORD wind_open(int Handle, WORD x, WORD y, WORD w, WORD h)
	{
	XMoveResizeWindow(display, GEMwins[Handle].win, x, y,
					calcW(Handle, w), calcH(Handle, h));
	GEMdoResize(Handle , w, h);
	GEMmakeActive(Handle);
	XMapSubwindows(display, GEMwins[Handle].win);
	XMapWindow(display, GEMwins[Handle].win);
	if (DEBUG) XFlush(display);
	}


int calcW(int Handle, int clientW)
	{
	return clientW + GEMwins[Handle].sideSpace;
	}

int calcH(int Handle, int clientH)
	{
	return clientH + GEMwins[Handle].topSpace + GEMwins[Handle].botSpace
				   + GEMwins[Handle].infSpace;
	}


Window createWindow(Window parent, int x, int y, int w, int h)
	{
	Window theWin;

	theWin = XCreateSimpleWindow(display, parent, x, y, w, h, 1,
						BlackPixel(display,screen_num),
						WhitePixel(display,screen_num));

	XSelectInput(display, theWin, StructureNotifyMask | 
						ExposureMask  | ButtonPressMask   |
						ButtonReleaseMask | PointerMotionMask );

	return theWin;
	}
